@@ -156,7 +156,6 @@ def _is_arg_name(s, index, node):
 # Elements here are in the form
 # (node class, predicate for start | None, predicate for end | None)
 NODES_REQUIRING_SOURCE = [
-    (nodes.AssignAttr, None, _is_attr_name),
     (nodes.AsyncFor, _keyword_search("async"), None),
     (nodes.AsyncFunctionDef, _keyword_search("async"), None),
     (nodes.AsyncWith, _keyword_search("async"), None),
@@ -204,8 +203,8 @@ def init_register_ending_setters(source_code):
 
     # Ad hoc transformations
     ending_transformer.register_transform(nodes.BinOp, _set_start_from_first_child)
-    ending_transformer.register_transform(nodes.ClassDef, _set_start_from_first_child)
-    ending_transformer.register_transform(nodes.FunctionDef, _set_start_from_first_child)
+    ending_transformer.register_transform(nodes.ClassDef, _set_start_from_first_decorator)
+    ending_transformer.register_transform(nodes.FunctionDef, _set_start_from_first_decorator)
     ending_transformer.register_transform(nodes.Tuple, _set_start_from_first_child)
     ending_transformer.register_transform(nodes.Arguments, fix_arguments(source_code))
     ending_transformer.register_transform(nodes.Slice, fix_slice(source_code))
@@ -292,7 +291,7 @@ def fix_subscript(source_code):
     """
 
     def _fix_end(node: nodes.Subscript) -> nodes.Subscript:
-        if isinstance(node.slice, nodes.Slice):
+        if isinstance(node.slice, (nodes.Slice, nodes.Tuple)):
             # In this case, the subscript node already contains the final ].
             return node
 
@@ -407,6 +406,15 @@ def _set_start_from_first_child(node):
     return node
 
 
+def _set_start_from_first_decorator(node):
+    """Set the start attributes of this node from its first child, if that child is a decorator."""
+    if getattr(node, "decorators"):
+        first_child = node.decorators
+        node.fromlineno = first_child.fromlineno
+        node.col_offset = first_child.col_offset
+    return node
+
+
 def set_from_last_child(node):
     """Populate ending locations for astroid node based on its last child.
 
@@ -478,7 +486,10 @@ def end_setter_from_source(source_code, pred, only_consumables=False):
     """
 
     def set_endings_from_source(node):
-        if not hasattr(node, "end_col_offset"):
+        # Tuple nodes have an end_col_offset that includes the end paren,
+        # but their col_offset does not include the start paren.
+        # To address this, we override the Tuple node's end_col_offset.
+        if not hasattr(node, "end_col_offset") or isinstance(node, nodes.Tuple):
             set_from_last_child(node)
 
         # Initialize counters. Note: we need to offset lineno,
@@ -645,43 +656,16 @@ def _add_parens(source_code):
 
 # Make this module a pylint plugin
 def register(linter):
+    """Patch linter to apply message transform with source code."""
     old_get_ast = linter.get_ast
 
-    def new_get_ast(filepath, modname):
-        ast = old_get_ast(filepath, modname)
-        with open(filepath) as f:
-            source_code = f.readlines()
-        ending_transformer = TransformVisitor()
-        register_transforms(source_code, ending_transformer)
-        ending_transformer.visit(ast)
+    def new_get_ast(filepath, modname, data):
+        ast = old_get_ast(filepath, modname, data)
+        if ast is not None:
+            with open(filepath, encoding="utf-8") as f:
+                source_code = f.readlines()
+            ending_transformer = init_register_ending_setters(source_code)
+            ending_transformer.visit(ast)
         return ast
 
     linter.get_ast = new_get_ast
-
-
-def register_transforms(source_code, obj):
-    # Check consistency of astroid-provided fromlineno and col_offset attributes.
-    for node_class in nodes.ALL_NODE_CLASSES:
-        obj.register_transform(
-            node_class,
-            fix_start_attributes,
-            lambda node: (
-                getattr(node, "fromlineno", None) is None
-                or getattr(node, "col_offset", None) is None
-            ),
-        )
-
-    # Ad hoc transformations
-    obj.register_transform(nodes.Arguments, fix_arguments(source_code))
-
-    for node_class in NODES_WITH_CHILDREN:
-        obj.register_transform(node_class, set_from_last_child)
-    for node_class in NODES_WITHOUT_CHILDREN:
-        obj.register_transform(node_class, set_without_children)
-
-    # Nodes where the source code must also be provided.
-    for node_class, start_pred, end_pred in NODES_REQUIRING_SOURCE:
-        if start_pred is not None:
-            obj.register_transform(node_class, start_setter_from_source(source_code, start_pred))
-        if end_pred is not None:
-            obj.register_transform(node_class, end_setter_from_source(source_code, end_pred))
